﻿-- JSONB.SQL
Drop Table If Exists tUtiliseJSONB CASCADE ;

-- Création de la table
Create Table tUtiliseJSONB (
	iId serial Constraint tUtiliseJSONB_pk PRIMARY KEY
	, jValeurJSONB JSONB
	) ;

Grant All On tUtiliseJSONB To "AtoutFox" ;	-- Pour l'utiliser depuis Fox en se connectant en tant que "AtoutFox"

-- Insertion de données
INSERT INTO tUtiliseJSONB (jValeurJSONB) VALUES
	('{"Nom": "Rencontres AtoutBox"
		, "Début": "15 mars 2018"
		, "Fin": "16 mars 2018"
		, "URL": "http://www.atoutfox.org/dates_rencontres_atoutfox_2018-programme.asp"
		}') ;

-- Données multidimensionnelles
INSERT INTO tUtiliseJSONB (jValeurJSONB) VALUES
	('{"Nom": "Rencontres AtoutBox"
		, "Début": "15 mars 2018"
		, "Fin": "16 mars 2018"
		, "URL": "http://www.atoutfox.org/dates_rencontres_atoutfox_2018-programme.asp"
		, "Sessions": [
			{"Titre": "API vulgarisation", "Auteur": "Gregory ADAM"}
			, {"Titre": "VFP sur le WEB", "Auteur": "Gilles LAJOT-SARTHOU"}
			, {"Titre": "noSQL en VFP", "Auteur": "Luc GILOT"}
			]
		}') ;

-- Vérification de la syntaxe JSON
-- INSERT INTO tUtiliseJSONB (jValeurJSONB) VALUES
-- 	('{"Nom": "Rencontres AtoutBox"
-- 		, "Début": "15 mars 2018"
-- 		, "Fin": "16 mars 2018"
-- 		, "URL": "http://www.atoutfox.org/dates_rencontres_atoutfox_2018-programme.asp"
-- 		, "Sessions": [
-- 			{"Titre": "API vulgarisation", "Auteur": "Gregory ADAM"}
-- 			, {"Titre": "VFP sur le WEB", "Auteur": "Gilles LAJOT-SARTHOU"}
-- 			, "Titre": "noSQL en VFP", "Auteur": "Luc GILOT"}
-- 			]
-- 		}') ;

		
-- Lecture des données
Select * From  tUtiliseJSONB ;
Select iId, jValeurJSONB->'Nom' AS Nom, jValeurJSONB->'Début' AS Début, jValeurJSONB->'Fin' AS Fin From tUtiliseJSONB ;
Select iId, jValeurJSONB->>'Nom' AS Nom, jValeurJSONB->>'Début' AS Début, jValeurJSONB->>'Fin' AS Fin From tUtiliseJSONB ;
Select iId, jValeurJSONB ? 'Nom' AS YaNom, jValeurJSONB ? 'Sessions' AS YaSessions, jValeurJSONB ? 'nom' AS YaNom2 From tUtiliseJSONB ;
Select jValeurJSONB->>'Sessions' From tUtiliseJSONB Where iId <= 3 ;
Select iId, jValeurJSONB#>'{Sessions, 1}', jValeurJSONB#>>'{Sessions, 2}' From tUtiliseJSONB Where iId <= 3 ;

-- Quelques fonctions : row_to_json
Select row_to_json((1, 'Un')) ;

Drop Table If Exists maTable ;
Create Table maTable(Id Int, Nom VarChar) ;
Insert Into MaTable Values (1, 'Un'), (2, 'Deux'), (3, 'Trois') ;
Select row_to_json(MaTable.*) From MaTable ;

With tmp As (
	Select 1 As Id, 'Un' As Nom
	Union
	Select 2, 'Deux'
	Union
	Select 3, 'Trois'
	)
	Select row_to_json(Tmp.*) From Tmp ;

-- Quelques fonctions : jsonb_each -> tous les champs du JSON dans des lignes
Select iId, jValeurJSONB->'Sessions' From tUtiliseJSONB Where iId = 3 ;
Select iId, jsonb_each(jValeurJSONB) From tUtiliseJSONB Where iId = 3 ;
Select iId, jsonb_each(jValeurJSONB) From tUtiliseJSONB Where iId < 10 ;

-- Quelques fonctions : jsonb_populate_record -> renormaliser
Drop Type If Exists monType1 ;

Create Type monType1 As (Id Int, Nom VarChar, URL VarChar) ;
Select iId, jsonb_populate_record(null::monType1, jValeurJSONB) From tUtiliseJSONB Where iId = 3 ;

Drop Type If Exists monType2 ;
Create Type monType2 As ("Id" Int, "Nom" VarChar, "URL" VarChar) ;
Select iId, jsonb_populate_record(null::monType2, jValeurJSONB) From tUtiliseJSONB Where iId = 3 ;

Select iId, (jsonb_populate_record(null::monType2, jValeurJSONB)).* From tUtiliseJSONB Where iId = 3 ;

Drop Type If Exists monType3 ;
Create Type monType3 As ("Id" Int, "Nom" VarChar, "URL" VarChar, "Inexistant" VarChar) ;
Select iId, (jsonb_populate_record(null::monType3, jValeurJSONB)).* From tUtiliseJSONB Where iId = 3 ;

Select iId, (jsonb_populate_record(null::monType3, jValeurJSONB)).* From tUtiliseJSONB ;

-- Quelques fonctions : jsonb_populate_recordset -> remettre les collections en lignes
Drop Type If Exists monType4 ;
Create Type monType4 As (Titre VarChar, Auteur VarChar) ;
Select iId, jsonb_populate_recordset(null::monType4, jValeurJSONB->'Sessions') From tUtiliseJSONB Where iId = 3 ;

Drop Type If Exists monType5 ;
Create Type monType5 As ("Titre" VarChar, "Auteur" VarChar, "Inexistant" VarChar) ;
Select iId, jsonb_populate_recordset(null::monType5, jValeurJSONB->'Sessions') From tUtiliseJSONB Where iId = 3 ;

Select iId, (jsonb_populate_recordset(null::monType3, jValeurJSONB->'Sessions')).* From tUtiliseJSONB Where iId = 3 ;

-- Mise à jour des données : opérateur ||, fusion de 2 JSONBs
Update tUtiliseJSONB set jValeurJSONB = jValeurJSONB || '{"Nom": "Rencontres AtoutFox"}' Where jValeurJSONB->>'Nom' = 'Rencontres AtoutBox' ;
Select * From tUtiliseJSONB ;

-- Ajout d'une propriété : idem
Update tUtiliseJSONB set jValeurJSONB = jValeurJSONB || '{"Lieu": "Hotel IBIS Lyon Perrache"}' Where iId = 1 ;
Select * From tUtiliseJSONB ;

-- Mise à jour des données : suppression de propriété
Update tUtiliseJSONB set jValeurJSONB = jValeurJSONB - 'Début' Where iId = 1 ;
Select * From tUtiliseJSONB ;

-- Insérer quelques lignes
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Insert Into tUtiliseJSONB (jValeurJSONB) Select jValeurJSONB || ('{"Id":"' || currval('tutiliseJSONB_iid_seq') + 1 || '"}')::JSONB From tUtiliseJSONB ;
Select Count(*) From tUtiliseJSONB ;
Select * From tUtiliseJSONB ;

-- Rechercher et indexer
Select * From tUtiliseJSONB Where jValeurJSONB->'Id' = '20000' ;			-- piège : -> renvoie JSONB
Select * From tUtiliseJSONB Where jValeurJSONB->>'Id' = '20000' ;			-- 150ms
Select iId From tUtiliseJSONB Where iId = 200000 ;
Select * From tUtiliseJSONB Where Not jValeurJSONB ? 'Id' ;				

CREATE INDEX idx_JSONB ON tUtiliseJSONB ((jValeurJSONB ->> 'Id'));		-- fonctionne : indexable sur le contenu, et pas très long
Select * From tUtiliseJSONB Where jValeurJSONB->>'Id' = '200000' ;		-- 12ms

Update vUtiliseJSONBNormalisée Set Nom = 'Rencontres AtoutFox 2' Where iId = 1 ;
